"""
Copyright (C) 2018 Intel Corporation

SPDX-License-Identifier: MIT
"""

import os

from tool import utils
from tool.generators_utils import (
    ADDITIONAL_ENV,
    ARCHITECTURE_DEFINE,
    BUILD_SCRIPT,
    CALL_ENV_SCRIPT_COMMAND,
    COMPILE_COMMAND_FORMAT,
    COMPILERS,
    COMPILERS_FLAGS,
    CPUID_FEATURES,
    CUSTOM_DISPATCHER_FILE,
    DISPATCHING_SCHEME_FORMAT,
    ENV_VAR,
    EXP_LIBS,
    FUNCTION_DISPATCHER,
    INCLUDE_STR,
    INIT_CHECK_FORMAT,
    LINK_COMMAND_FORMAT,
    LINKER_FLAGS,
    LINKERS,
    LINUX,
    MAIN_FILE,
    OPENMP,
    RENAME_FORMAT,
    RENAME_HEADER_SCHEME,
    RETURN_VALUES,
    SET_ENV_COMMAND,
    SYS_LIBS_PATH,
    UNINITIALIZED_FEATURES,
    UNINITIALIZED_FEATURES_DEFINE,
    WINDOWS,
)


def main_file_generator():
    return MAIN_FILE[utils.HOST_SYSTEM].format(package_type=utils.CONFIGS[utils.PACKAGE].type.lower())


def create_windows_export_file(export_file, functions_list):
    """
    Creates export file for Windows

    :param export_file: object that is returned by open function
    :param functions_list: list of functions for dynamic library
    """
    export_file.write("EXPORTS\n\n")
    export_file.writelines(map(lambda x: f"{x}\n", functions_list))


def create_linux_export_file(export_file, functions_list):
    """
    Creates Linux export file

    :param export_file:
    :param functions_list:
    :return:
    """
    export_file.writelines(map(lambda x: f"EXTERN({x})\n", functions_list))
    export_file.write(
        r"""
VERSION
{{
    {{
        global:
{functions_list}
        local:* ;
    }};
}}""".format(
            functions_list="".join(map(lambda x: f"        {x};\n", functions_list))
        )
    )


EXPORT_GENERATORS = {
    WINDOWS: create_windows_export_file,
    LINUX: create_linux_export_file,
}


def custom_dispatcher_generator(function, dispatcher_type: utils.DispatcherType = utils.DispatcherType.DYNAMIC):
    package = utils.CONFIGS[utils.PACKAGE]
    arch = utils.CONFIGS[utils.ARCHITECTURE]
    include_lines = INCLUDE_STR.format(header_name=package.type.lower() + ".h")

    dispatcher = ""
    dispatcher += func_dispatcher_generator(arch, function, dispatcher_type)

    ippe = utils.DOMAINS[utils.IPP]["ippe"]
    if ippe in package.functions[utils.IPP].keys() and function in package.functions[utils.IPP][ippe]:
        ippe_header_name = "ippe.h"
        if package.dir_layout == utils.DirLayout.NEW:
            ippe_header_name = "ipp/" + ippe_header_name
        include_lines += "\n" + INCLUDE_STR.format(header_name=ippe_header_name)

    if dispatcher_type == utils.DispatcherType.STATIC:
        uninitialized_features_define = UNINITIALIZED_FEATURES_DEFINE.format(
            uninitialized_features=UNINITIALIZED_FEATURES[package.type]
        )
    else:
        uninitialized_features_define = ""

    return CUSTOM_DISPATCHER_FILE.format(
        include_lines=include_lines,
        architecture=ARCHITECTURE_DEFINE[arch],
        features=CPUID_FEATURES[package.type],
        uninitialized_features_define=uninitialized_features_define,
        dispatcher=dispatcher,
    )


def rename_header_generator(functions_list):
    package = utils.CONFIGS[utils.PACKAGE]
    prefix = utils.CONFIGS[utils.PREFIX]

    package_type_lower = package.type.lower()
    package_type_upper = package.type.upper()
    defs_header = "defs.h" if package.type == utils.IPP or package.type == utils.IPPCP else "base.h"
    defs_header_name = package_type_lower + defs_header
    if package.dir_layout == utils.DirLayout.NEW:
        defs_header_name = f"{package_type_lower}/{defs_header_name}"
    defines_header_statement = INCLUDE_STR.format(header_name=defs_header_name)
    rename_statements = ""
    for function in functions_list:
        declaration = package.declarations[function].replace(function, prefix + function)
        rename_statements += RENAME_FORMAT.format(declaration=declaration, function=function, prefix=prefix)
    return RENAME_HEADER_SCHEME.format(
        prefix=package_type_upper,
        defines_header_statement=defines_header_statement,
        rename_statements=rename_statements,
    )


def func_dispatcher_generator(arch, function, dispatcher_type: utils.DispatcherType = utils.DispatcherType.DYNAMIC):
    package_type = utils.CONFIGS[utils.PACKAGE].type
    declarations = utils.CONFIGS[utils.PACKAGE].declarations[function]
    function_with_prefix = utils.CONFIGS[utils.PREFIX] + function
    ippfun = declarations.replace("IPPAPI", "IPPFUN").replace(function, function_with_prefix)

    args = utils.get_match(utils.FUNCTION_NAME_REGEX, declarations, "args").split(",")
    args = [utils.get_match(utils.ARGUMENT_REGEX, arg, "arg") for arg in args]
    args = ", ".join(args)

    ret_type = utils.get_match(utils.FUNCTION_NAME_REGEX, declarations, "ret_type")
    ret_value = utils.get_dict_value(RETURN_VALUES, ret_type)
    if ret_value:
        return_keyword = "return "
    else:
        return_keyword = ""

    ippapi = ""
    dispatching_scheme = ""
    for cpu in utils.CPUID.keys():
        if cpu not in utils.CONFIGS[utils.CUSTOM_CPU_SET]:
            continue

        cpuid = utils.CPUID[cpu]
        cpu_code = utils.CPU[cpu][arch]

        function_with_cpu_code = f"{cpu_code}_{function}"
        ippapi += declarations.replace(function, function_with_cpu_code) + "\n"

        dispatching_scheme += DISPATCHING_SCHEME_FORMAT.format(
            cpuid=cpuid, return_keyword=return_keyword, function=function_with_cpu_code, args=args
        )

    if dispatcher_type == utils.DispatcherType.STATIC:
        init_check = INIT_CHECK_FORMAT.format(
            package_type=package_type.lower(), return_keyword=return_keyword, function=function_with_prefix, args=args
        )
    else:
        init_check = ""

    return_statement = f"\n    return {ret_value};" if ret_value else ""

    return FUNCTION_DISPATCHER.format(
        ippapi=ippapi,
        ippfun=ippfun,
        package_type=package_type.lower(),
        dispatching_scheme=dispatching_scheme,
        init_check=init_check,
        return_statement=return_statement,
    )


def build_script_generator():
    """
    Generates script for building custom dynamic library
    :return: String that represents script
    """
    host = utils.HOST_SYSTEM
    configs = utils.CONFIGS

    package = configs[utils.PACKAGE]
    output_path = configs[utils.OUTPUT_PATH]

    arch = configs[utils.ARCHITECTURE]
    thread = configs[utils.THREAD_MODE]
    tl = configs[utils.THREADING_LAYER]

    root_type = utils.IPPROOT if package.type == utils.IPP else utils.IPPCRYPTOROOT

    if package.env_script:
        force_flag = ""
        if "setvars" in package.env_script:
            force_flag = "--force"

        env_commands = CALL_ENV_SCRIPT_COMMAND[host].format(
            env_script=package.env_script, arch=arch, force_flag=force_flag
        )
    else:
        env_commands = SET_ENV_COMMAND[host].format(env_var=root_type, path=package.root)
    if ADDITIONAL_ENV[host]:
        env_commands += "\n" + ADDITIONAL_ENV[host]

    compiler = COMPILERS[host]

    cmp_flags = COMPILERS_FLAGS[host][arch]
    if tl == OPENMP and host == WINDOWS:
        cmp_flags += " /openmp"

    c_files = utils.MAIN_FILE_NAME
    if configs[utils.CUSTOM_CPU_SET]:
        c_files += " " + os.path.join("custom_dispatcher", arch, "*.c")

    compile_command = COMPILE_COMMAND_FORMAT[host].format(
        compiler=compiler, cmp_flags=cmp_flags, root_type=root_type, c_files=c_files
    )

    linker = LINKERS[host]
    link_flags = LINKER_FLAGS[host][arch]
    custom_library = utils.LIB_PREFIX[host] + configs[utils.CUSTOM_LIBRARY_NAME]
    export_file = utils.EXPORT_FILE[host]

    ipp_libraries = package.libraries[arch][thread]
    if tl:
        ipp_libraries = package.libraries[arch][tl] + ipp_libraries
    ipp_libraries = [lib.replace(package.root, ENV_VAR[host].format(env_var=root_type)) for lib in ipp_libraries]
    ipp_libraries = " ".join(f'"{lib}"' for lib in ipp_libraries)

    exp_libs = EXP_LIBS[host][thread]
    if tl and EXP_LIBS[host][tl] not in exp_libs:
        exp_libs += " " + EXP_LIBS[host][tl]

    sys_libs_path = SYS_LIBS_PATH[host][arch]

    link_command = LINK_COMMAND_FORMAT[host].format(
        linker=linker,
        link_flags=link_flags,
        custom_library=custom_library,
        export_file=export_file,
        ipp_libraries=ipp_libraries,
        exp_libs=exp_libs,
        sys_libs_path=sys_libs_path,
    )

    return BUILD_SCRIPT[host].format(
        architecture=arch,
        threading=thread.lower(),
        output_path=output_path,
        custom_library=custom_library,
        env_commands=env_commands,
        compile_command=compile_command,
        link_command=link_command,
    )



"""
Copyright (C) 2018 Intel Corporation

SPDX-License-Identifier: MIT
"""

import os
import sys
from argparse import ArgumentParser, RawTextHelpFormatter

import tool.package
from tool import utils
from tool.core import build, generate_script

if __name__ == "__main__":
    utils.set_host_system()

    ipp_package_name = utils.PACKAGE_NAME[utils.IPP]
    ippcp_package_name = utils.PACKAGE_NAME[utils.IPPCP]

    args_parser = ArgumentParser(formatter_class=RawTextHelpFormatter)

    args_parser.add_argument(
        "-c",
        "--console",
        help="Does nothing. The option is left for compatibility with previous versions of Custom Library Tool",
    )

    args_parser.add_argument(
        "-g",
        "--generate",
        help="Generate build script without building custom dynamic library",
        default="none",
        const="dynamic",
        nargs="?",
        choices=("static", "dynamic"),
    )
    args_parser.add_argument(
        "-n", "--name", help="Name of custom dynamic library", default=utils.CONFIGS[utils.CUSTOM_LIBRARY_NAME]
    )
    args_parser.add_argument(
        "-p", "--output_path", help="Path to output directory", default=utils.CONFIGS[utils.OUTPUT_PATH]
    )
    args_parser.add_argument(
        "-root",
        help=f"Path to specified {ipp_package_name}\nor {ippcp_package_name} package",
    )
    args_parser.add_argument(
        "-f",
        "--functions",
        help="Functions that has to be in dynamic library (appendable)",
        nargs="+",
        metavar="FUNCTION",
    )
    args_parser.add_argument("-ff", "--functions_file", help="Load custom functions list from text file")
    args_parser.add_argument(
        "-arch", "--architecture", help="Target architecture", choices=utils.ARCHITECTURES, default="intel64"
    )
    args_parser.add_argument(
        "-tl",
        "--threading_layer_type",
        help="Build dynamic library with selected threading layer type",
        choices=utils.TL_TYPES,
    )
    ipp_supported_cpus = " ".join(utils.SUPPORTED_CPUS[utils.IPP][utils.INTEL64][utils.HOST_SYSTEM])
    ippcp_supported_cpus = " ".join(utils.SUPPORTED_CPUS[utils.IPPCP][utils.INTEL64][utils.HOST_SYSTEM])
    args_parser.add_argument(
        "-d",
        "--custom_dispatcher",
        help=rf"""Build dynamic library with custom dispatcher.
Set of CPUs can be any combination of the following:
{ipp_package_name}:
    Intel(R) 64 architecture - {ipp_supported_cpus}
{ippcp_package_name}:
    Intel(R) 64 architecture - {ippcp_supported_cpus}""",
        nargs="+",
        metavar="CPU",
    )
    args_parser.add_argument("--prefix", help="Rename functions in custom dispatcher with specified prefix", default="")

    args = args_parser.parse_args()

    print("Custom Library Tool")

    if args.root:
        root = args.root
        if not os.path.exists(root):
            sys.exit(f"Error: specified package path {args.root} doesn't exist")
    else:
        root = tool.package.get_package_path()
        if not os.path.exists(root):
            sys.exit(
                f"Error: cannot find {ipp_package_name} or {ippcp_package_name} package. "
                "Please, specify IPPROOT or IPPCRYPTOROOT by using -root option"
            )

    package = tool.package.Package(root)
    print(f"Current package: {package.name}")

    architecture = args.architecture
    thread_mode = utils.SINGLE_THREADED
    threading_layer_type = args.threading_layer_type

    error = package.errors[architecture][thread_mode]
    if error:
        sys.exit(f"Error: {error}")
    if threading_layer_type:
        error = package.errors[architecture][threading_layer_type]
        if error:
            sys.exit(f"Error: {error}")

    custom_cpu_set = []
    if args.custom_dispatcher:
        for cpu in args.custom_dispatcher:
            if cpu not in utils.SUPPORTED_CPUS[package.type][architecture][utils.HOST_SYSTEM]:
                full_package_name = f"{utils.PACKAGE_NAME[package.type]} {utils.HOST_SYSTEM} {architecture}"
                sys.exit(f"Error: {cpu} isn't supported for {full_package_name}")
        custom_cpu_set = args.custom_dispatcher
    prefix = args.prefix

    custom_library_name = args.name
    output_path = os.path.abspath(args.output_path)
    if not os.path.exists(output_path):
        os.makedirs(output_path, 0o744)

    functions_list = []
    if args.functions_file:
        with open(args.functions_file, "r") as functions_file:
            functions_list += functions_file.read().splitlines()
    if args.functions:
        functions_list += args.functions
    if not functions_list:
        # If neither `-f` nor `-ff` is specified, all functions from the current package are included by default:
        # * Intel(R) Integrated Performance Primitives (excluding Threading Layer functions) or
        # * Intel(R) Cryptography Primitives Library
        if package.type in package.functions:
            print("Neither -f nor -ff options are specified, so all functions are included in custom library.")
            package_functions = package.functions[package.type]
            for domain_functions_list in package_functions.values():
                functions_list += domain_functions_list
        else:
            sys.exit("Please, specify functions that has to be in dynamic library by using -f or -ff options")

    utils.set_configs_dict(
        package=package,
        functions_list=functions_list,
        architecture=architecture,
        thread_mode=thread_mode,
        threading_layer_type=threading_layer_type,
        custom_library_name=custom_library_name,
        output_path=output_path,
        custom_cpu_set=custom_cpu_set,
        prefix=prefix,
    )

    if args.generate != "none":
        dispatcher_type = utils.DispatcherType(args.generate)
        success = generate_script(dispatcher_type)
        print("Generation", "completed!" if success else "failed!")
    else:
        success = build()

    if not success:
        exit(1)



"""
Copyright (C) 2018 Intel Corporation

SPDX-License-Identifier: MIT
"""

import os
from collections import defaultdict
from pathlib import Path

from tool import utils


class Package:
    def __init__(self, path):
        self.root = path

        self.headers = utils.nested_dict_init()
        self.functions = utils.nested_dict_init()
        self.declarations = defaultdict()

        self.libraries = utils.nested_dict_init()
        self.features = utils.nested_dict_init()
        self.errors = utils.nested_dict_init()

        self.functions_without_dispatcher = []

        self.set_type_and_layout()
        self.set_env_script()

        self.set_headers_functions_declarations_dicts()
        self.set_libraries_features_errors_dicts(self.type, utils.THREAD_MODES)
        self.set_libraries_features_errors_dicts(utils.THREADING_LAYER, utils.TL_TYPES)

        self.package_validation()
        self.set_name()

    def set_type_and_layout(self):
        self.type = utils.IPP
        self.dir_layout = utils.DirLayout.NEW
        self.headers_dir = os.path.join(self.root, "include")
        for package_type in [utils.IPP, utils.IPPCP]:
            package_type_lower = package_type.lower()
            for dir_layout in utils.DirLayout:
                dir_layout_subdir = package_type_lower if dir_layout == utils.DirLayout.NEW else ""
                headers_dir = os.path.join(self.headers_dir, dir_layout_subdir)
                main_header = os.path.join(headers_dir, f"{package_type_lower}.h")
                if os.path.exists(main_header):
                    self.type = package_type
                    self.dir_layout = dir_layout
                    self.headers_dir = headers_dir
                    return

    def set_name(self):
        self.name = "None"
        version = self.get_version()

        if not self.broken:
            self.name = f"{utils.PACKAGE_NAME[self.type]} Version {version}"

    def get_version(self):
        version = ""
        header = os.path.join(self.headers_dir, "ippversion.h")

        for line in utils.get_lines_from_file(header):
            version = utils.get_match(utils.VERSION_REGEX, line, "ver")
            if version:
                break
        if not version:
            version = "None"

        version = self.parse_version_string(version, header)

        return version

    def parse_version_string(self, version, header):
        while True:
            macros = utils.get_match(utils.STR_MACROS_REGEX, version, "macros")
            if not macros:
                break

            macros_value = ""
            for line in utils.get_lines_from_file(header):
                if macros in line:
                    macros_value = line.split(macros, 1)[1].strip()
                    break

            version = version.replace(f"STR({macros})", macros_value)

        while True:
            c_string = utils.get_match(utils.C_STRING_REGEX, version, "string")
            if not c_string:
                break

            c_string_value = utils.get_match(utils.C_STRING_VALUE_REGEX, c_string, "value")
            version = version.replace(c_string, c_string_value)

        return version

    def set_env_script(self):
        paths_to_search = []
        batch_extension = utils.BATCH_EXTENSIONS[utils.HOST_SYSTEM]

        component_dir = Path(self.root).parent
        if component_dir.name == self.type.lower():
            oneapi_dir = component_dir.parent
            paths_to_search.append(os.path.join(oneapi_dir, f"setvars{batch_extension}"))
        paths_to_search.append(os.path.join(self.root, "env", f"vars{batch_extension}"))

        self.env_script = utils.get_first_existing_path_in_list(paths_to_search)

    def set_headers_functions_declarations_dicts(self):
        if not os.path.exists(self.headers_dir):
            return

        headers = [header for header in os.listdir(self.headers_dir) if ".h" in header]

        for header in headers:
            domain_type, domain = self.get_type_and_domain(header)
            if not domain:
                continue

            functions_list = []
            incomplete_function = ""
            for line in utils.get_lines_from_file(os.path.join(self.headers_dir, header)):
                if incomplete_function:
                    self.declarations[incomplete_function] += " " + " ".join(line.split())
                    incomplete_function = self.get_function_if_incomplete(incomplete_function)
                    continue

                function = utils.get_match(utils.FUNCTION_NAME_REGEX, line, "function_name")
                if not function:
                    continue
                functions_list.append(function)
                self.declarations[function] = " ".join(line.split())
                incomplete_function = self.get_function_if_incomplete(function)

            if not functions_list:
                continue

            self.headers[domain_type][domain] = header

            domain_name = utils.DOMAINS[domain_type][domain]
            if domain_name not in self.functions[domain_type]:
                self.functions[domain_type][domain_name] = functions_list
            else:
                self.functions[domain_type][domain_name] += functions_list

            if header == "ippcpdefs.h" or domain_type == utils.THREADING_LAYER or domain == "ippcore":
                self.functions_without_dispatcher += functions_list

    def set_libraries_features_errors_dicts(self, domains_type, thread_types):
        host = utils.HOST_SYSTEM

        for arch in utils.ARCHITECTURES:
            for thread_type in thread_types:
                self.libraries[arch][thread_type] = []
                path_to_libraries = self.get_path_to_libraries(arch, thread_type)

                for domain in utils.DOMAINS[domains_type].keys():
                    lib_name = (
                        utils.LIB_PREFIX[host]
                        + domain
                        + utils.STATIC_LIB_POSTFIX[host]
                        + utils.LIB_POSTFIX[thread_type]
                        + utils.STATIC_LIB_EXTENSION[host]
                    )

                    lib_path = os.path.join(path_to_libraries, lib_name)

                    if os.path.exists(lib_path):
                        self.libraries[arch][thread_type].append(lib_path)
                    elif not domain == "ippe":
                        self.libraries[arch][thread_type].clear()
                        break

                self.features[arch][thread_type] = bool(self.libraries[arch][thread_type])
                self.errors[arch][thread_type] = self.check_headers_and_libs(arch, domains_type, thread_type)

    def package_validation(self):
        self.broken = True
        self.error_message = self.errors[utils.INTEL64][utils.SINGLE_THREADED]

        for arch in utils.ARCHITECTURES:
            for thread in utils.THREAD_MODES:
                if self.features[arch][thread]:
                    self.broken = False
                    return

    def get_type_and_domain(self, header):
        domain_type = self.type if "_tl" not in header else utils.THREADING_LAYER
        for domain in utils.DOMAINS[domain_type].keys():
            if domain in header:
                return domain_type, domain
        return "", ""

    def get_function_if_incomplete(self, function):
        if self.declarations[function].count("(") != self.declarations[function].count(")"):
            return function
        return ""

    def get_path_to_libraries(self, arch, thread_type):
        lib_arch_subdir = utils.LIB_ARCH_SUBDIR[self.dir_layout]
        lib_thread_type_subdir = utils.LIB_THREAD_TYPE_SUBDIR[self.dir_layout][thread_type]
        return os.path.join(self.root, lib_arch_subdir, lib_thread_type_subdir)

    def check_headers_and_libs(self, arch, domains_type, thread_type):
        for domain in utils.DOMAINS[domains_type].keys():
            if domains_type not in self.headers.keys() or (
                domain not in self.headers[domains_type].keys() and not domain == "ippe"
            ):
                return f"Broken package - cannot find header files for {domains_type} functions"

        if not self.libraries[arch][thread_type]:
            return f"Cannot find {thread_type} libraries for {arch} architecture"

        return ""


def get_package_path():
    current_path = os.path.realpath(__file__)
    paths_to_search = [
        utils.get_match(utils.PATH_TO_PACKAGE_REGEX[utils.DirLayout.NEW], current_path, "path"),
        utils.get_match(utils.PATH_TO_PACKAGE_REGEX[utils.DirLayout.OLD], current_path, "path"),
        utils.get_env(utils.IPPROOT),
        utils.get_env(utils.IPPCRYPTOROOT),
    ]

    return utils.get_first_existing_path_in_list(paths_to_search)



#=========================================================================
# Copyright (C) 2017 Intel Corporation
#
# Licensed under the Apache License,  Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law  or agreed  to  in  writing,  software
# distributed under  the License  is  distributed  on  an  "AS IS"  BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the  specific  language  governing  permissions  and
# limitations under the License.
#=========================================================================

#
# Intel® Cryptography Primitives Library
#

import re
import sys
import os
import hashlib
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-i', '--header', action='store', required=True, help='Intel Cryptography Primitives Library dispatcher will be generated for functions in Header')
parser.add_argument('-o', '--out-directory', action='store', required=True, help='Output folder for generated files')
parser.add_argument('-l', '--cpu-list', action='store', required=True, help='Actual CPU list: semicolon separated string')
parser.add_argument('-c', '--compiler', action='store', help='Compiler') # is not used
args = parser.parse_args()
Header = args.header
OutDir = args.out_directory
cpulist = args.cpu_list.split(';')

headerID= False      ## Header ID define to avoid multiple include like: #if !defined( __IPPCP_H__ )

from gen_disp_common import readNextFunction

HDR= open( Header, 'r' )
h= HDR.readlines()
HDR.close()

## keep filename only
(incdir, Header)= os.path.split(Header)

## original header name to declare external functions as internal for dispatcher
OrgH= Header

isFunctionFound = True
curLine = 0
FunName = ""
FunArg = ""

while (isFunctionFound == True):

  result = readNextFunction(h, curLine, headerID)

  curLine         = result['curLine']
  FunName         = result['FunName']
  FunArg          = result['FunArg']
  isFunctionFound = result['success']

  if (isFunctionFound == True):
    ##################################################
    ## create dispatcher files: C file with inline asm
    ##################################################
    filename = "jmp_{}_{}".format(FunName, hashlib.sha512(FunName.encode('utf-8')).hexdigest()[:8])

    DISP= open( os.sep.join([OutDir, filename + ".asm"]), 'w' )

    for cpu in cpulist:
       DISP.write("extern "+cpu+"_"+FunName+"\n")

    DISP.write("extern ippcpJumpIndexForMergedLibs\n")
    DISP.write("extern ippcpSafeInit\n\n")

    DISP.write("segment data\n\n")

    DISP.write("    DQ in_"+FunName+"\n")
    DISP.write(FunName+"_arraddr:\n")

    for cpu in cpulist:
       DISP.write("    DQ "+cpu+"_"+FunName+"\n")

    DISP.write("""

segment text

global {FunName}

in_{FunName}:
   {endbr64}
   call    ippcpSafeInit
   align 16
{FunName}:
   {endbr64}
   movsxd  rax, dword [rel ippcpJumpIndexForMergedLibs]
   lea     r10, [rel {FunName}_arraddr]
   mov     r10, qword [r10+rax*8]
   jmp     r10

""".format(FunName=FunName, endbr64='db 0xf3, 0x0f, 0x1e, 0xfa'))

    DISP.close()



#=========================================================================
# Copyright (C) 2017 Intel Corporation
#
# Licensed under the Apache License,  Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law  or agreed  to  in  writing,  software
# distributed under  the License  is  distributed  on  an  "AS IS"  BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the  specific  language  governing  permissions  and
# limitations under the License.
#=========================================================================

#
# Intel(R) Cryptography Primitives Library
#

import re
import sys
import os
import hashlib
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-i', '--header', action='store', required=True, help='Intel Cryptography Primitives Library dispatcher will be generated for functions in Header')
parser.add_argument('-o', '--out-directory', action='store', required=True, help='Output folder for generated files')
parser.add_argument('-l', '--cpu-list', action='store', required=True, help='Actual CPU list: semicolon separated string')
parser.add_argument('-c', '--compiler', action='store', help='Compiler') # is not used
args = parser.parse_args()
Header = args.header
OutDir = args.out_directory
cpulist = args.cpu_list.split(';')

headerID= False      ## Header ID define to avoid multiple include like: #if !defined( __IPPCP_H__ )

from gen_disp_common import readNextFunction

HDR= open( Header, 'r' )
h= HDR.readlines()
HDR.close()


## keep filename only
(incdir, Header)= os.path.split(Header)

## original header name to declare external functions as internal for dispatcher
OrgH= Header

isFunctionFound = True
curLine = 0
FunName = ""
FunType = ""
FunArg = ""

while (isFunctionFound == True):

  result = readNextFunction(h, curLine, headerID)

  curLine         = result['curLine']
  FunType         = result['FunType']
  FunName         = result['FunName']
  FunArg          = result['FunArg']
  isFunctionFound = result['success']

  if (isFunctionFound == True):

      ##################################################
      ## create dispatcher files: C file with inline asm
      ##################################################
      DISP= open( os.sep.join([OutDir, "jmp_"+FunName+"_" + hashlib.sha512(FunName.encode('utf-8')).hexdigest()[:8] + ".c"]), 'w' )

      DISP.write("""#include "ippcpdefs.h" """)

      DISP.write("\n\ntypedef " + FunType + "(*IPP_PROC)(void);\n\n")
      DISP.write("extern int ippcpJumpIndexForMergedLibs;\n")
      DISP.write("extern IppStatus IPP_CALL ippcpInit();\n\n")

      DISP.write("extern " + FunType + " IPP_CALL in_"+FunName+FunArg+";\n")

      for cpu in cpulist:
         DISP.write("extern " + FunType + " IPP_CALL "+cpu+"_"+FunName+FunArg+";\n")

      DISP.write("static IPP_PROC arraddr[] =\n{{\n	(IPP_PROC)in_{}".format(FunName))

      for cpu in cpulist:
         DISP.write(",\n	(IPP_PROC)"+cpu+"_"+FunName+"")

      DISP.write("\n};")

      DISP.write("""
#ifdef _MSC_VER
#pragma warning(disable: 4100) // for MSVC, unreferenced param
#endif
#undef  IPPAPI
#define IPPAPI(type,name,arg) __declspec(naked) type __stdcall name arg
IPPAPI({FunType}, {FunName},{FunArg})
{{
  __asm {{mov eax, dword ptr ippcpJumpIndexForMergedLibs}}
  __asm {{mov eax, arraddr[eax*4+4]}}
  __asm {{jmp eax}}
}}

IPPAPI({FunType}, in_{FunName},{FunArg})
{{
  __asm {{call ippcpInit}}
  __asm {{mov eax, dword ptr ippcpJumpIndexForMergedLibs }}
  __asm {{mov eax, arraddr[eax*4+4]}}
  __asm {{jmp eax}}
}}
""".format(FunType=FunType, FunName=FunName, FunArg=FunArg))

      DISP.close()




"""
Copyright (C) 2018 Intel Corporation

SPDX-License-Identifier: MIT
"""

import enum
import os
import re
import sys
from collections import defaultdict
from platform import system

DirLayout = enum.Enum("DirLayout", ["NEW", "OLD"])


class DispatcherType(enum.Enum):
    STATIC = "static"
    DYNAMIC = "dynamic"


WINDOWS = "Windows"
LINUX = "Linux"
HOST_SYSTEM = None
SUPPORTED_SYSTEMS = [WINDOWS, LINUX]

INTEL64 = "intel64"
ARCHITECTURES = [INTEL64]

LIB_ARCH_SUBDIR = {
    DirLayout.NEW: "lib",
    DirLayout.OLD: os.path.join("lib", "intel64"),
}

SINGLE_THREADED = "single-threaded"
THREAD_MODES = [SINGLE_THREADED]

TBB = "tbb"
OPENMP = "openmp"
TL_TYPES = [TBB, OPENMP]

PATH_TO_PACKAGE_REGEX = {
    DirLayout.NEW: r"(?P<path>.*)[\\/]opt[\\/]ipp(cp)?[\\/]tools[\\/].*",
    DirLayout.OLD: r"(?P<path>.*)[\\/]tools[\\/].*",
}

VERSION_REGEX = r".*VERSION_STR\s*(?P<ver>.*)\s*"
STR_MACROS_REGEX = r".*STR\((?P<macros>\S*)\).*"
C_STRING_REGEX = r'.*(\S|^)(?P<string>\s*".*"\s*)(\S|$).*'
C_STRING_VALUE_REGEX = r'.*"(?P<value>.*)".*'
FUNCTION_NAME_REGEX = (
    r"IPPAPI\s*\(\s*(?P<ret_type>.*?)\s*," r"\s*(?P<function_name>\S*)\s*," r"\s*\(?(?P<args>.*?)\s*\)?\s*\)?\s*$"
)
ARGUMENT_REGEX = r".*\W*\w+\W*\s+\W*(?P<arg>[^\W\d]+\w*)\W*?"

CUSTOM_LIBRARY_NAME = "Custom library name"
BUILD_SCRIPT_NAME = "Build script name"
OUTPUT_PATH = "Output path"
FUNCTIONS_LIST = "Functions list"
PACKAGE = "Package"
ARCHITECTURE = "Architecture"
THREAD_MODE = "Thread mode"
THREADING_LAYER = "Threading layer"
CUSTOM_CPU_SET = "Custom CPU set"
PREFIX = "Prefix"

CONFIGS = {
    CUSTOM_LIBRARY_NAME: "custom_library",
    BUILD_SCRIPT_NAME: "",
    OUTPUT_PATH: ".",
    FUNCTIONS_LIST: [],
    PACKAGE: "",
    ARCHITECTURE: "",
    THREAD_MODE: "",
    THREADING_LAYER: "",
    CUSTOM_CPU_SET: [],
    PREFIX: "",
}

IPP = "IPP"
IPPCP = "IPPCP"

IPPROOT = "IPPROOT"
IPPCRYPTOROOT = "IPPCRYPTOROOT"

PACKAGE_NAME = {
    IPP: "Intel(R) Integrated Performance Primitives",
    IPPCP: "Intel(R) Cryptography Primitives Library",
}

DOMAINS = {
    IPP: {
        "ippcc": "Color Conversion",
        "ippcv": "Computer Vision",
        "ippdc": "Data Compression",
        "ippe": "Embedded Functionality",
        "ippi": "Image Processing",
        "ipps": "Signal Processing",
        "ippvm": "Vector Math",
        "ippcore": "Core",
    },
    IPPCP: {"ippcp": "Cryptography"},
    THREADING_LAYER: {
        "ippcc": "Color Conversion TL",
        "ippcv": "Computer Vision TL",
        "ippi": "Image Processing TL",
        "ippcore": "Core TL",
    },
}

SSE42 = "sse42"
AVX2 = "avx2"
AVX512BW = "avx512bw"
AVX512IFMA = "avx512ifma"
AVX10_2 = "avx10.2"

CPU = {
    SSE42: {INTEL64: "y8"},
    AVX2: {INTEL64: "l9"},
    AVX512BW: {INTEL64: "k0"},
    AVX512IFMA: {INTEL64: "k1"},
    AVX10_2: {INTEL64: "d1"},
}

SUPPORTED_CPUS = {
    IPP: {
        INTEL64: {
            WINDOWS: [SSE42, AVX2, AVX512BW, AVX10_2],
            LINUX: [SSE42, AVX2, AVX512BW, AVX10_2],
        },
    },
    IPPCP: {
        INTEL64: {
            WINDOWS: [SSE42, AVX2, AVX512BW, AVX512IFMA, AVX10_2],
            LINUX: [SSE42, AVX2, AVX512BW, AVX512IFMA, AVX10_2],
        },
    },
}

CPUID = {
    AVX10_2: "APX_FEATURES",
    AVX512IFMA: "AVX3I_FEATURES",
    AVX512BW: "AVX3X_FEATURES",
    AVX2: "ippCPUID_AVX2",
    SSE42: "ippCPUID_SSE42",
}

LIB_THREAD_TYPE_SUBDIR = {
    DirLayout.NEW: {
        SINGLE_THREADED: "",
        TBB: "",
        OPENMP: "",
    },
    DirLayout.OLD: {
        SINGLE_THREADED: "",
        TBB: os.path.join("tl", TBB),
        OPENMP: os.path.join("tl", OPENMP),
    },
}

LIB_PREFIX = {WINDOWS: "", LINUX: "lib"}

LIB_POSTFIX = {SINGLE_THREADED: "", TBB: "_tl_tbb", OPENMP: "_tl_omp"}

STATIC_LIB_POSTFIX = {WINDOWS: "mt", LINUX: ""}

STATIC_LIB_EXTENSION = {WINDOWS: ".lib", LINUX: ".a"}

DYNAMIC_LIB_EXTENSION = {WINDOWS: ".dll", LINUX: ".so"}

BUILD_SCRIPT_NAME_FORMAT = {
    WINDOWS: "build_{name}_{arch}.bat",
    LINUX: "build_{name}_{arch}.sh",
}

BATCH_EXTENSIONS = {WINDOWS: ".bat", LINUX: ".sh"}

MAIN_FILE_NAME = "main.c"
RENAME_HEADER_NAME = "rename.h"

EXPORT_FILE = {WINDOWS: "export.def", LINUX: "export.def"}

PROJECT_EXTENSION = ".cltproj"

HAVE_PACKAGE = "package..."
HAVE_FUNCTIONS = "functions that has to be in dynamic library... "

ENABLE_GENERATION_RULES = {
    HAVE_PACKAGE: True,
    HAVE_FUNCTIONS: True,
}


def set_host_system():
    host_system = system()

    if host_system not in SUPPORTED_SYSTEMS:
        sys.exit(f"Error: Custom Library Tool doesn't support OS {host_system}")

    global HOST_SYSTEM
    HOST_SYSTEM = host_system


def set_configs_dict(
    package,
    functions_list,
    architecture,
    thread_mode,
    threading_layer_type,
    custom_library_name=CONFIGS[CUSTOM_LIBRARY_NAME],
    build_script_name=CONFIGS[BUILD_SCRIPT_NAME],
    output_path=CONFIGS[OUTPUT_PATH],
    custom_cpu_set=CONFIGS[CUSTOM_CPU_SET],
    prefix=CONFIGS[PREFIX],
):
    CONFIGS[CUSTOM_LIBRARY_NAME] = custom_library_name
    CONFIGS[OUTPUT_PATH] = output_path
    CONFIGS[FUNCTIONS_LIST] = functions_list
    CONFIGS[PACKAGE] = package
    CONFIGS[ARCHITECTURE] = architecture
    CONFIGS[THREAD_MODE] = thread_mode
    CONFIGS[THREADING_LAYER] = threading_layer_type
    CONFIGS[CUSTOM_CPU_SET] = custom_cpu_set
    CONFIGS[PREFIX] = prefix

    if not build_script_name:
        build_script_name = BUILD_SCRIPT_NAME_FORMAT[HOST_SYSTEM].format(name=custom_library_name, arch=architecture)
    CONFIGS[BUILD_SCRIPT_NAME] = build_script_name


def get_first_existing_path_in_list(paths_list):
    for path in paths_list:
        if os.path.exists(path):
            return path
    return ""


def get_lines_from_file(file_path):
    if os.path.exists(file_path):
        with open(file_path, "r") as file:
            return file.readlines()
    else:
        return []


def get_env(env_var):
    return os.environ[env_var] if os.getenv(env_var) and os.path.exists(os.environ[env_var]) else ""


def get_match(regex, string, group):
    return re.match(regex, string).group(group) if re.compile(regex).match(string) else ""


def nested_dict_init():
    return defaultdict(lambda: defaultdict())


def walk_dict(dictionary):
    for key, value in dictionary.items():
        if isinstance(value, dict) or isinstance(value, defaultdict):
            for entire in walk_dict(value):
                yield (key,) + entire
        elif isinstance(value, list):
            for elem in value:
                yield key, elem
        else:
            yield key, value


def get_dict_value(dictionary, key):
    return dictionary[key] if key in dictionary.keys() else dictionary["default"]



# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

SOURCE_DIR = os.path.dirname(__file__)
LATEX_DIR = os.path.join(SOURCE_DIR, '_latex')
PREAMBLE_FILE = os.path.join(LATEX_DIR, 'preamble.tex')
TITLE_PAGE_FILE = os.path.join(LATEX_DIR, 'title_page.tex')

BUILD_TYPE = os.getenv("BUILD_TYPE")

# -- Project information -----------------------------------------------------


project = u'Intel® Cryptography Primitives Library'
copyright = u'2022, Intel Corporation'
author = u'Intel'

# The short X.Y version
version = u''
# The full version, including alpha/beta/rc tags
release = u'v1.1.0'


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
nbsphinx_allow_errors = True

nbsphinx_execute_arguments = [
    "--InlineBackend.figure_formats={'svg', 'pdf'}",
    "--InlineBackend.rc={'figure.dpi': 150}",
]

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.imgmath',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['main/_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
#master_doc = 'main/title_main'
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build','_themes', '**.ipynb_checkpoints']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Syntax highlighting for the :: directive
highlight_language = 'cpp'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#

html_theme = 'sphinx_book_theme'

#otc_tcs_sphinx_theme
version = current_version = "latest"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.

# Replace value of 'canonical_url' below with actual location of doc website.
# TODO: Write user-setup section to customize dictionary output below.

html_theme_options = {
  'navigation_depth': 4,
  'collapse_navigation': False
}

html_copy_source = False


# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = ['_themes']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "Documentation for Intel® oneAPI Programming Guide"

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.

html_logo = '_images_pdfly/logo-classicblue-400px.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = '_images_pdfly/favicon.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
#html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
# html_extra_path = ['_html_extra']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants =

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_domain_indices = False

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
html_split_index = True

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink =

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'hu', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'ru', 'sv', 'tr'
# html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# Now only 'ja' uses this config value
#html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
# html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.

#TODO: Add option for html_context in user-setup
# htmlhelp_basename = ''





# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# -- sphinx2dita configuration -----------------------------------------------

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'sphinx-infodevdoc'


# -- Options for LaTeX output ------------------------------------------------

#latex_engine = 'xelatex'
#PDF_TITLE = 'Information Development Template'
#
#with open(PREAMBLE_FILE, 'r', encoding='utf-8') as f:
#    PREAMBLE = f.read()
#
#with open(TITLE_PAGE_FILE, 'r', encoding='utf-8') as f:
#    TITLE_PAGE = f.read().replace('<PDF_TITLE>', PDF_TITLE)
#
#
#latex_elements = {
#    # The paper size ('letterpaper' or 'a4paper').
#    #
#    'extraclassoptions': 'openany,oneside',
#    'babel' : '\\usepackage[english]{babel}',
#    'papersize': 'a4paper',
#    'releasename':" ",
#    # Sonny, Lenny, Glenn, Conny, Rejne, Bjarne and Bjornstrup
#    # 'fncychap': '\\usepackage[Lenny]{fncychap}',
#    'fncychap':  '',
#    #'fontpkg': '\\usepackage{amsmath,amsfonts,amssymb,amsthm}',
#
#    'figure_align':'htbp',
#    # The font size ('10pt', '11pt' or '12pt').
#    #
#    'pointsize': '12pt',
#
#    # Additional stuff for the LaTeX preamble.
#    #
#    'preamble': PREAMBLE,
#
#    'maketitle': TITLE_PAGE,
#    # Latex figure (float) alignment
#    #
#    # 'figure_align': 'htbp',
#    'sphinxsetup': \
#        'hmargin={0.7in,0.7in}, vmargin={1in,1in}, \
#        verbatimwithframe=true, \
#        TitleColor={rgb}{0,0.686,0.941}, \
#        HeaderFamily=\\rmfamily\\bfseries, \
#        InnerLinkColor={rgb}{0,0.686,0.941}, \
#        OuterLinkColor={rgb}{0,0.686,0.941}',
#
#    'tableofcontents':' '
#}
#
#latex_logo = '_latex/intel_logo.png'
## Grouping the document tree into LaTeX files. List of tuples
## (source start file, target name, title,
##  author, documentclass [howto, manual, or own class]).
#latex_documents = [
#    (master_doc, 'sphinx-infodev.tex', u'sphinx-infodev Documentation',
#     u'Intel', 'manual'),
#]

#breathe_projects = {                                       #todd-mod
#    project: "../doxygen/xml"
#}
#breathe_default_project = project

# Setup the exhale extension
#exhale_args = {                                            #todd-mod
#    # These arguments are required
#    "containmentFolder":     "./api",
#    "rootFileName":          "library_root.rst",
#    "rootFileTitle":         "Library API",
#    "doxygenStripFromPath":  "..",
#    "fullApiSubSectionTitle": 'Full API'
#}


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'sphinx-infodev', u'sphinx-infodev Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'sphinx-infodev', u'sphinx-infodev Documentation',
     author, 'sphinx-infodev', 'One line description of project.',
     'Miscellaneous'),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']


# -- Extension configuration -------------------------------------------------

# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
# intersphinx_mapping = {'https://docs.python.org/': None}

# -- Options for todo extension ----------------------------------------------

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

